/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"
#ifndef ALIGNED2_HPP_
#define ALIGNED2_HPP_

template<typename T, uint align2 = 64>
class Aligned2 {
    void *  rawMem;
    T *     objMem;
    explicit Aligned2(Aligned2 const &) {}
    Aligned2 & operator=(Aligned2 const &) { return *this; }
    void clean();
public:
    Aligned2(bool const doAllocAndConstruct = false);
    ~Aligned2();
    void allocAndConstruct();
    operator T*const () const { return objMem; }
    T * const operator->() const { return objMem; }
    T & operator * () const { return *objMem; }
};

template<typename T, uint align2>
Aligned2<T, align2>::Aligned2(bool const doAllocAndConstruct) 
        : rawMem(0),objMem(0) {
    if (doAllocAndConstruct) {
        allocAndConstruct();
    }
}

template<typename T, uint align2>
void Aligned2<T, align2>::allocAndConstruct() {
    if ( rawMem ) {
        return;
    }
    rawMem = malloc( sizeof(T) + align2 - 1 );
    SOUT("sizeof(T) = " << sizeof(T));
    if ( !rawMem ) {
        throw std::bad_alloc();
    }
    try {
        objMem = new(   
                        reinterpret_cast<T*>( 
                            (reinterpret_cast<ULONG_PTR>(rawMem) + align2 - 1) 
                                & (0xffffffffffffffffUL - align2 + 1) 
                        )
                     ) T();
    } catch (...) {
        clean();
        throw;
    }
}

template<typename T, uint align2>
Aligned2<T, align2>::~Aligned2() {
    clean();
    #ifndef NDEBUG
        // how to write to the damn output window of visual c++ without managed crap???
       // SOUT("DUMPING MEM LEAKS..\n");
        _CrtDumpMemoryLeaks();
        // SOUT("..DUMP FINISHED" << std::endl);
    #endif
}

template<typename T, uint align2>
void Aligned2<T, align2>::clean() {
    if ( objMem ) {
        objMem->~T();
        objMem = 0;
    }
    if ( rawMem ) {
        free(rawMem);
        rawMem = 0;
    }
}

#endif
