/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"

#ifndef MSGBOX_HPP_
#define MSGBOX_HPP_

#include "globals.hpp"
#include "Singleton.hpp"
#include "events.hpp"

template<typename Functor = binfunctor>
class MsgBox : public Singleton<MsgBox<Functor> > {
    typedef typename Functor::first_argument_type  functorA1_t;
    typedef typename Functor::second_argument_type functorA2_t;
    typedef typename Functor::result_type          functorReturn_t;
    ctstr       text_;
    ctstr       title_;
    int         flags_;
    int         ret4action_;
    Functor     functor_;
    // stored as type is required for constants but sucks in other cases
    functorA1_t functorA1_;
    functorA2_t functorA2_;
    MsgBox();
    friend Singleton<MsgBox<Functor> >;
public:   
    ~MsgBox();
    void set(ctstr text, ctstr title = g_msg_mpFssErr,
             int flags = MB_OK | MB_ICONERROR,
             int ret4action = -1 /* noop */,
             Functor functor = Functor(0),
             functorA1_t functorA1 = 0, // requires an int assignable arg..sucks a bit
             functorA2_t functorA2 = 0);
    void show() const;
};

template<typename Functor>
MsgBox<Functor>::MsgBox() : Singleton(),text_(0),title_(0),flags_(0),
        ret4action_(-1),functor_(Functor(0)),functorA1_(),functorA2_() {
    g_msgbox = this;
}

template<typename Functor>
MsgBox<Functor>::~MsgBox() {}

template<typename Functor>
void MsgBox<Functor>::set(ctstr text, ctstr title, int flags, int ret4action,
                          Functor functor,
                          functorA1_t functorA1,
                          functorA2_t functorA2) {
    text_ =       text;
    title_ =      title;
    flags_ =      flags;
    ret4action_ = ret4action;
    functor_ =    functor;
    functorA1_ =  functorA1;
    functorA2_ =  functorA2;

    SimConnect_TransmitClientEvent(g_scHandle, 0, ev_msgbox, 0, ngidNormal, 
                                   SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
}

template<typename Functor>
void MsgBox<Functor>::show() const {
    int ret;
    ret = MessageBox(g_winHWND, text_, title_, flags_);
    if ( ret == ret4action_ ) {
        SOUT("MessageBox returned val 4 action");
        functor_(functorA1_, functorA2_);
    }
    SOUT("MessageBox returned " << ret);
}

#endif
