/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"
#ifndef OPTDATA_HPP_
#define OPTDATA_HPP_

#include "myFunctional.hpp"

struct inputDev {
    inputDev() : axes_(0),buttons_(0),activeAxes_(0) { clear(); }
    void                    clear();
    void                    BitorActiveAxes(uint axeBit) { activeAxes_ |= axeBit; }
    uint                    getActiveAxes() const { return activeAxes_; }
    GUID const &            getGUID() const { return guidInstance_; }
    std::tstring const &    getNameProduct() const { return nameProduct_; }               
    void                    setButtonsAndAxis(DWORD bu, DWORD ax);
    void                    setGUID(GUID const & guid) { guidInstance_ = guid; }
    void                    setNameProduct(cpctstr name) { nameProduct_.assign(name); }
private:
    std::tstring    nameProduct_;
    GUID            guidInstance_;
    short           axes_;
    short           buttons_;
    uint            activeAxes_;
};

namespace {
struct VersionNumber {
    VersionNumber(uchar major, uchar minor, short patch) : 
        major_(major),minor_(minor),patch_(patch) {}
private:
    uchar major_,
          minor_;
    short patch_;
};
} // ns anon

#define AXISSTRSIZE 4

class optData {
public:
    optData();
    ~optData();
    cpccstr                 getAxisStr(bool const left = true) const;
    cpctstr                 getAxis(bool const left = true) const;
    cpctstr                 getDeadzonePercent(bool left = true) const;
    int                     getDeadzone(bool const left = true) const;
    int                     getDzSub(bool const left = true) const;
    int                     getDzRem(bool const left = true) const;
    GUID const &            getGUID() const;
    bool                    getInverted(bool const left = true) const;
    bool                    areOptionsSet() const;
    void                    readOptFile();
    void                    writeOptFile();
    void                    setActDevNr(char const n);
    void                    setAll(GUID const & in_actDev, cpctstr in_Ldead,
                                   cpctstr in_Rdead, int const in_actDevNr,
                                   cpctstr in_Laxis, cpctstr in_Raxis,
                                   int const in_Linverted, int const in_Rinverted );
    void                    setNameProd(std::tstring const & instr);
    void                    setOptionsSet(bool const yes);
private:
    template<typename streamT, typename funcT>
    void        doReadWrite( streamT & stream, funcT bf, bool read );
    void        setDeadzone(cpctstr str, bool const left = true);

    static TCHAR const  optFile_[];
    static std::tstring optFilePath_;
    // change the version only if needed (like stucture change and not if just an update has occured)
    VersionNumber       version_;
    GUID                actDev_;
    TCHAR               Laxis_[AXISSTRSIZE];
    TCHAR               Raxis_[AXISSTRSIZE];
    int                 Ldeadzone_;
    int                 Rdeadzone_;
    int                 LdzSub_;
    int                 RdzSub_;
    int                 LdzRem_;
    int                 RdzRem_;
    char                actDevNr_;
    bool                Linverted_;        
    bool                Rinverted_;
    uchar               optionsSet_:1;
    uchar               canLoadSave_:1;
    uchar               padding:6;
};

#endif
