/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krüger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"

#include "dispatchProc.hpp"
#include "events.hpp"
#include "globals.hpp"
#include "optDialog.hpp"
#include "Resource.h"
#include "enDisable.hpp"
#include "MsgBox.hpp"

#ifndef NDEBUG
struct dataBrakes {
    float left;
    float right;
};
#endif
struct dataThrottle {
    float pct_;
};

namespace {
__forceinline void pedalSender(int const evPedal, DWORD const inData, bool const inverted, bool const left);
__forceinline int dzCorrectedPedal(bool const left, int org);
__forceinline void toggleKeyBrake();
__forceinline void turnKeyBrakeOffIfNeeded();
__forceinline void correctThrottleIfNeeded(float pct);
} // ns anon

void CALLBACK DispatchAlot(SIMCONNECT_RECV * pData, DWORD  /* pDataSize */, void * /* myPtr */) {
    SIMCONNECT_RECV_EVENT const * const ev = RC(SIMCONNECT_RECV_EVENT const * const, pData);
    switch (ev->dwID) {
    case SIMCONNECT_RECV_ID_SIMOBJECT_DATA:
        switch ( RC(SIMCONNECT_RECV_SIMOBJECT_DATA *, pData)->dwRequestID ) {
        case in_throttle:
                correctThrottleIfNeeded(RC(dataThrottle*, &RC(SIMCONNECT_RECV_SIMOBJECT_DATA *, pData)->dwData)->pct_ );
                break;
#ifndef NDEBUG   
            case data_brakes:
                cout << "\ndata = brake left: " << RC(dataBrakes*, &RC(SIMCONNECT_RECV_SIMOBJECT_DATA *, pData)->dwData)->left;
                cout << "  brake right: " << RC(dataBrakes*, &RC(SIMCONNECT_RECV_SIMOBJECT_DATA *, pData)->dwData)->right;
                break;
#endif
        }
        break;
    case SIMCONNECT_RECV_ID_EVENT:
    case SIMCONNECT_RECV_ID_EVENT_FILENAME:
        switch (ev->uEventID) {
        case in_pedalL:
            pedalSender(ev_pedalL, ev->dwData, g_options->getInverted(), true);
            break;
        case in_pedalR:
            pedalSender(ev_pedalR, ev->dwData, g_options->getInverted(false), false);
            break;
        case ev_brake:
            toggleKeyBrake();
            break;
        case menu_enable:
            if (g_enabled) {
                disable();
            } else {
                enable();
            }
            break;
        case menu_options:
        case ev_options:
            SimConnect_SetSystemState(g_scHandle, "DialogMode", 1, 0.0f, 0);
            SimConnect_RequestSystemState(g_scHandle, rq_dialogMode, "DialogMode");
            break;
        case ev_msgbox:
            SimConnect_SetSystemState(g_scHandle, "DialogMode", 1, 0.0f, 0);
            SimConnect_RequestSystemState(g_scHandle, rq_msgbox, "DialogMode");
            break;
        case menu_about:
            MsgBox<>::singleton().set(g_aboutStr, _T("about mpFSS.brakeFix"), MB_OK | MB_ICONINFORMATION);
            break;
#ifndef NDEBUG
        default:
            SOUT("unknown event...the id was " << ev->uEventID << " dwID = " << ev->dwID);
            break;
#endif
        }
        break;
    case SIMCONNECT_RECV_ID_SYSTEM_STATE:
        switch( RC(SIMCONNECT_RECV_SYSTEM_STATE const * const, ev)->dwRequestID ) {
        case rq_msgbox:
            g_msgbox->show();
            SimConnect_SetSystemState(g_scHandle, "DialogMode", 0, 0.0f, 0);
            break;
        case rq_dialogMode:
            try {
                g_optsDialog = new optDialog();
            } catch (std::exception &) {
                MsgBox<>::singleton().set(errMsg::get(2));
                return;
            }
            g_optsDialog->show();
            DELETE(g_optsDialog)
            SimConnect_SetSystemState(g_scHandle, "DialogMode", 0, 0.0f, 0);
            if ( g_enabled ) {
                turnKeyBrakeOffIfNeeded();
                disable();
                if ( g_options->areOptionsSet() ) {
                    enable();
                }
            }
            break;
        }
        break;
    //case SIMCONNECT_RECV_ID_EXCEPTION:
    // do sth or not
    //    break;
#ifndef NDEBUG
    default:
        SOUT("recieved an unknown event in dispatch..id=" << pData->dwID << 
            " size=" << pData->dwSize);
        break;
#endif
    }
}

/*
            not inverted    inverted
    max     -32768           32768
    min      32768          -32768
*/

namespace {
#define TRANS2BRAKES(val__) SimConnect_TransmitClientEvent(g_scHandle, SIMCONNECT_OBJECT_ID_USER, \
        ev_pedalL, val__, ngidPedals, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY); \
    SimConnect_TransmitClientEvent(g_scHandle, SIMCONNECT_OBJECT_ID_USER, \
        ev_pedalR, val__, ngidPedals, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);

__forceinline void pedalSender(int const evPedal, DWORD const inData, bool const inverted, bool const left) {
    int pedalState = SC(int, inData);
    if ( inverted == false ) {
        pedalState *= -1;
    }
    SOUT("got a pedal " << evPedal << " event:" << pedalState);
    if ( pedalState < g_options->getDeadzone(left) || pedalState == PEDALMIN ) {
        if ( g_brakesOn ) {
            SOUT("below dz: keyBrake on applying max brake " << evPedal);
            TRANS2BRAKES(BRAKEMAX)
            return;
        } else {
            SOUT("below dz: setting brake " << evPedal << " off");
            SimConnect_TransmitClientEvent(g_scHandle, SIMCONNECT_OBJECT_ID_USER,
                evPedal, SC(DWORD, BRAKEMIN), ngidPedals, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
        }
    } else {
        SOUT("applying brake " << evPedal << " with " << (pedalState >> 1)
            << " (" << ((pedalState >> 1) + 16384) / 32767.0f << " pct)");

        turnKeyBrakeOffIfNeeded();
        pedalState = dzCorrectedPedal(left, pedalState);

        SOUT("applying brake(corr) " << evPedal << " with " << pedalState
            << " (" << (pedalState + 16384) / 32767.0f << " pct)");

        SimConnect_TransmitClientEvent(g_scHandle, SIMCONNECT_OBJECT_ID_USER,
            evPedal, pedalState, ngidPedals, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
    }
}

__forceinline int dzCorrectedPedal(bool const left, int org) {
    float pct;
    if ( org == PEDALMIN ) { return BRAKEMIN; }
    org += PEDALMAX;
    org -= g_options->getDzSub(left);
    pct = SC(float, org) / g_options->getDzRem(left);
    org = SC(int, PEDALMAXCOMBINED * pct - PEDALMAX);
    org = org >> 1;
    if ( org > BRAKEMAX ) {
        org = BRAKEMAX;
        SOUT("over max.. reducing to max");
    }
    return org;    
}

__forceinline void toggleKeyBrake() {
    if ( g_brakesOn ) {
        g_brakesOn = false;
        TRANS2BRAKES(SC(DWORD, BRAKEMIN))
        SOUT("releasing key brakes");
    } else {
        g_brakesOn = true;
        TRANS2BRAKES(BRAKEMAX)
        SOUT("applying key brakes");
    }
}
// more of the same code but saving an if statement aka else if(toggle) in toggleKeyBrakeIfNeeded
__forceinline void turnKeyBrakeOffIfNeeded() {
    if ( g_brakesOn ) {
        g_brakesOn = false;
        TRANS2BRAKES(SC(DWORD, BRAKEMIN))
        SOUT("releasing key brakes");
    }
}

__forceinline void correctThrottleIfNeeded(float const pct) {
    SOUT("throttle: " << pct );
    if ( pct < 2.0f && !(pct < 0.0f) ) {
        SOUT("setting throttle to idle");
        SimConnect_TransmitClientEvent(g_scHandle, SIMCONNECT_OBJECT_ID_USER,
                ev_throttle, SC(DWORD, 0), ngidPedals, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
    }
}
} // ns anon
