/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krüger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"

#include "dxi.hpp"
#include "optData.hpp"
#include "mpexception.hpp"
#include "globals.hpp"
#include "MsgBox.hpp"

#ifdef FILEID__
#undef FILEID__
#endif
#define FILEID__ dxi_cpp
//namespace {
//IDirectInput8 * l_dinput =    0;
//} // ns anon
BOOL __stdcall EnumDevicesCB(LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef);

typedef BOOL (__stdcall * EnumDevCB)(LPCDIDEVICEINSTANCE, LPVOID);

namespace {
class DInput8 {
    IDirectInput8 * dInput8_;
    void *          fwdPtr_;
    explicit DInput8(DInput8 const &) {}
    DInput8 & operator=(DInput8 const &) { return *this; }
public:
    DInput8(HINSTANCE hInst, int dInputVersion = DIRECTINPUT_VERSION, 
            IUnknown * comPtr = 0) : dInput8_(0),fwdPtr_(0) {
        HRESULT hr;
        hr = DirectInput8Create(hInst, dInputVersion, IID_IDirectInput8,
                                RC(void**, &dInput8_), comPtr);
        if ( hr != DI_OK ) {
            throw MPEXCEPTIONMSG(GetLastError(), errMsg::get(3));
        }
        // TODO delete me
//        throw MPEXCEPTIONMSG(GetLastError(), errMsg::get(3));
    }
    ~DInput8() { dInput8_->Release(); }
    void enumDevs(  EnumDevCB cbf, void * fwdPtr = 0, 
                    DWORD devType = DI8DEVCLASS_GAMECTRL,
                    DWORD devFlags = DIEDFL_ATTACHEDONLY) {
        HRESULT hr;
        fwdPtr_ = fwdPtr ? fwdPtr : const_cast<DInput8*>(this);
        hr = dInput8_->EnumDevices(devType, cbf, fwdPtr_, devFlags);
        if ( hr != DI_OK ) { 
            throw MPEXCEPTIONMSG(GetLastError(), errMsg::get(25));
        }
    }
    void * const getFwdPtr() const { return fwdPtr_; }
    IDirectInput8 * const direct() const { return dInput8_; }
};

class DInputDev {
    IDirectInputDevice8 * diDev_;
    explicit DInputDev(DInputDev const &) {}
    DInputDev & operator=(DInputDev const &) { return *this; }
public:
    DInputDev() : diDev_(0) {}
    ~DInputDev() { if (diDev_) { diDev_->Release(); } }
    void init(DInput8 const * const dinput8, GUID const & guid) {
        HRESULT hr = dinput8->direct()->CreateDevice(guid, &diDev_, NULL);
        if ( hr != DI_OK ) { 
            throw MPEXCEPTIONMSG(GetLastError(), errMsg::get(4));
        }
    }
    IDirectInputDevice8 * const direct() const { return diDev_; }
};

class DInputDevAcquired {
    DInputDev const & dinputDev_;
    explicit DInputDevAcquired(DInputDevAcquired const & rhs) : dinputDev_(rhs.dinputDev_) {}
    DInputDevAcquired & operator=(DInputDevAcquired const &) { return *this; }
public:
    DInputDevAcquired(DInputDev const & dinputDev) : dinputDev_(dinputDev) {
        HRESULT hr = dinputDev_.direct()->Acquire();
        if ( hr != DI_OK ) { 
            throw MPEXCEPTIONMSG(GetLastError(), errMsg::get(5)); 
        }
    }
    ~DInputDevAcquired() { dinputDev_.direct()->Unacquire(); }
    void getDevState(DWORD const size, void * const jstate) const {
        HRESULT hr = dinputDev_.direct()->GetDeviceState(size, jstate);
        if ( hr != DI_OK ) {
            throw MPEXCEPTIONMSG(GetLastError(), errMsg::get(6));
        }
    }
    void getDevCaps(DIDEVCAPS * const devcaps) const {
        HRESULT hr = dinputDev_.direct()->GetCapabilities(devcaps);
        if ( hr != DI_OK ) {
            throw MPEXCEPTIONMSG(GetLastError(), errMsg::get(7));
        }
        // TODO delete me
//        throw MPEXCEPTIONMSG(GetLastError(), errMsg::get(7));
    }
};

struct CbCom {
    std::vector<inputDev> * iDevs_;
    mpexception *           exTrans_;
    DInput8 const *         dinput8_;
    bool                    errInCB_;
    char                    padding[3];
};

} // ns anon

void enumDevices(std::vector<inputDev> & inputDevs) {
    mpexception exce( SC(DWORD, -1) );
    CbCom cbCom;
    cbCom.iDevs_ =   &inputDevs;
    cbCom.errInCB_ = false;
    cbCom.exTrans_ = &exce;

    DInput8 dinput8(g_hInst);
    cbCom.dinput8_ = &dinput8;
    dinput8.enumDevs(&EnumDevicesCB, &cbCom);
    if ( cbCom.errInCB_ ) {
        throw mpexception(exce);
    }
}

BOOL __stdcall EnumDevicesCB(LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef) {
    CbCom * const cbCom =   RC(CbCom*, pvRef);
    inputDev iDev;
    DInputDev dinputDev;
    DIDEVCAPS diDev8caps =  { 0 };
    DIJOYSTATE2 diJstate2 = { 0 };
    diDev8caps.dwSize =     sizeof(DIDEVCAPS);
    
    iDev.setGUID(lpddi->guidInstance);
    try {
        iDev.setNameProduct(lpddi->tszProductName);
        dinputDev.init(cbCom->dinput8_, iDev.getGUID());
        dinputDev.direct()->SetCooperativeLevel(g_winHWND, DISCL_BACKGROUND | DISCL_NONEXCLUSIVE);
        dinputDev.direct()->SetDataFormat(&c_dfDIJoystick2);
        DInputDevAcquired dinputDevAcquired(dinputDev);
        dinputDevAcquired.getDevState(sizeof(DIJOYSTATE2), &diJstate2);
        dinputDevAcquired.getDevCaps(&diDev8caps);
        iDev.setButtonsAndAxis(diDev8caps.dwButtons, diDev8caps.dwAxes);
        if (diJstate2.lX) { iDev.BitorActiveAxes(BIT1); }
        if (diJstate2.lY) { iDev.BitorActiveAxes(BIT2); }
        if (diJstate2.lZ) { iDev.BitorActiveAxes(BIT3); }
        if (diJstate2.lRx) { iDev.BitorActiveAxes(BIT4); }
        if (diJstate2.lRy) { iDev.BitorActiveAxes(BIT5); }
        if (diJstate2.lRz) { iDev.BitorActiveAxes(BIT6); }
        cbCom->iDevs_->push_back(iDev);
    } catch (mpexception & e) {
        *(cbCom->exTrans_) = e;
        cbCom->errInCB_ = true;
        return DIENUM_STOP;
    } catch (std::exception &) {
        *(cbCom->exTrans_) = MPEXCEPTIONMSG(GetLastError(), errMsg::get(8));
        cbCom->errInCB_ = true;
        return DIENUM_STOP;
    }
    return DIENUM_CONTINUE;
}

//bool enumDevices(std::vector<inputDev> & inputDevs) {
//    bool ret = true;
//    DirectInput8Create(g_hInst, DIRECTINPUT_VERSION, IID_IDirectInput8,
//                        (void**)&l_dinput, NULL);
//    if (!l_dinput) {
//        MessageBox(g_winHWND, _T("Error creating direct input interface"), 
//                    g_msg_mpFssErr, MB_OK | MB_ICONERROR);
//        ret = false;
//    } else {
//        if ( l_dinput->EnumDevices(DI8DEVCLASS_GAMECTRL, &EnumDevicesCB, &inputDevs,
//            DIEDFL_ATTACHEDONLY) != S_OK ) {
//                ret = false;
//        }
//        l_dinput->Release();
//        l_dinput = 0;
//    }
//    return ret;
//}

//BOOL __stdcall EnumDevicesCB(LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef) {
//    std::vector<inputDev> * const inputDevs = 
//            reinterpret_cast<std::vector<inputDev> * const>(pvRef);
//    BOOL ret = DIENUM_CONTINUE;
//    inputDev iDev;
//    IDirectInputDevice8 * diDev8;
//    DIDEVCAPS diDev8caps = {0};
//    DIJOYSTATE2 diJstate2 = { 0 };
//    diDev8caps.dwSize = sizeof(DIDEVCAPS);
//    try {
//        iDev.setNameProduct(lpddi->tszProductName);
//    } catch (std::exception &) {
//        MessageBox(g_winHWND, _T("Error getting product Name"), 
//                    g_msg_mpFssErr, MB_OK | MB_ICONERROR);
//        return FALSE;
//    }
//    iDev.setGUID(lpddi->guidInstance);
//    if (l_dinput->CreateDevice(lpddi->guidInstance, &diDev8, NULL) != DI_OK) {
//        MessageBox(g_winHWND, _T("Error creating direct input device"), 
//                    g_msg_mpFssErr, MB_OK | MB_ICONERROR);
//        return FALSE;
//    }
//    diDev8->SetCooperativeLevel(g_winHWND, DISCL_BACKGROUND | DISCL_NONEXCLUSIVE);
//    diDev8->SetDataFormat(&c_dfDIJoystick2);
//    if ( diDev8->Acquire() != DI_OK ) {
//        MessageBox(g_winHWND, _T("Error could not aquire joystick"), 
//                   g_msg_mpFssErr, MB_OK | MB_ICONERROR);
//        ret = FALSE;
//    } else {
//        if ( diDev8->GetDeviceState(sizeof(DIJOYSTATE2), &diJstate2) == DI_OK ) {
//            if (diJstate2.lX) { iDev.BitorActiveAxes(BIT1); }
//            if (diJstate2.lY) { iDev.BitorActiveAxes(BIT2); }
//            if (diJstate2.lZ) { iDev.BitorActiveAxes(BIT3); }
//            if (diJstate2.lRx) { iDev.BitorActiveAxes(BIT4); }
//            if (diJstate2.lRy) { iDev.BitorActiveAxes(BIT5); }
//            if (diJstate2.lRz) { iDev.BitorActiveAxes(BIT6); }
//            diDev8->GetCapabilities(&diDev8caps);
//            iDev.setButtonsAndAxis(diDev8caps.dwButtons, diDev8caps.dwAxes);
//            try {
//                inputDevs->push_back(iDev);
//            } catch (std::exception &) {
//                MessageBox(g_winHWND, _T("Error saving temporary device"), 
//                    g_msg_mpFssErr, MB_OK | MB_ICONERROR);
//                ret = FALSE;
//            }
//        } else {
//            MessageBox(g_winHWND, _T("Error could not get joystick state"), 
//                        g_msg_mpFssErr, MB_OK | MB_ICONERROR);
//            ret = FALSE;
//        }
//        diDev8->Unacquire();
//    }
//    diDev8->Release();
//    return ret;
//}