/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krüger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"

#include "enDisable.hpp"
#include "globals.hpp"
#include "events.hpp"
#include "optDialog.hpp"
#include "dispatchProc.hpp"
#include "dxi.hpp"
#include "addMenus.hpp"
#include "mpexception.hpp"
#include "MsgBox.hpp"
#include "transCevent.hpp"

void disable() {
    SimConnect_SetInputGroupState(g_scHandle, ngidPedals, SIMCONNECT_STATE_OFF);
    SimConnect_ClearInputGroup(g_scHandle, ngidPedals);

    SimConnect_ClearNotificationGroup(g_scHandle, ngidHighest);
    SimConnect_RequestDataOnSimObject(g_scHandle, in_throttle, in_throttle,
                SIMCONNECT_OBJECT_ID_USER, SIMCONNECT_PERIOD_NEVER,
                SIMCONNECT_DATA_REQUEST_FLAG_CHANGED, 0, 0, 0);
    g_enabled = false;
    updateSubMenus();
}

void enable() {
    inputDevVec availDevs;
    inputDevVec::const_iterator act;
    inputDevVec::const_iterator end;
    char devFoundNr =   0;
    bool devFound =     false;

    if ( g_options->areOptionsSet() ) {
        try {
            availDevs.reserve(4);
            enumDevices(availDevs);
        } catch (mpexception & e) {
            e.what(true);
            return;
        } catch (std::exception &) {
            MsgBox<>::singleton().set(errMsg::get(9));
            return;
        }
        end = availDevs.end();
        for ( act = availDevs.begin(); act != end; ++act, ++devFoundNr ) {
            if ( act->getGUID() == g_options->getGUID() ) {
                devFound = true;
                break;
            }
        }
        if ( devFound ) {
            g_options->setActDevNr(devFoundNr);
            SimConnect_MapInputEventToClientEvent(g_scHandle, ngidPedals, 
                g_options->getAxisStr(), in_pedalL);
            SimConnect_MapInputEventToClientEvent(g_scHandle, ngidPedals, 
                g_options->getAxisStr(false), in_pedalR);
            SimConnect_SetInputGroupState(g_scHandle, ngidPedals, SIMCONNECT_STATE_ON);
            SimConnect_SetInputGroupPriority(g_scHandle, ngidPedals, 
                SIMCONNECT_GROUP_PRIORITY_HIGHEST);
            SimConnect_AddClientEventToNotificationGroup(g_scHandle, ngidHighest, in_pedalL, false);
            SimConnect_AddClientEventToNotificationGroup(g_scHandle, ngidHighest, in_pedalR, false);

            SimConnect_AddClientEventToNotificationGroup(g_scHandle, ngidHighest, ev_brake, false);
            SimConnect_SetNotificationGroupPriority(g_scHandle, ngidHighest, 
                SIMCONNECT_GROUP_PRIORITY_HIGHEST);
            SimConnect_TransmitClientEvent(g_scHandle, SIMCONNECT_OBJECT_ID_USER,
                ev_pedalL, SC(DWORD, BRAKEMIN), ngidPedals, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
            SimConnect_TransmitClientEvent(g_scHandle, SIMCONNECT_OBJECT_ID_USER,
                ev_pedalR, SC(DWORD, BRAKEMIN), ngidPedals, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
            SimConnect_RequestDataOnSimObject(g_scHandle, in_throttle, in_throttle,
                SIMCONNECT_OBJECT_ID_USER, SIMCONNECT_PERIOD_SIM_FRAME,
                SIMCONNECT_DATA_REQUEST_FLAG_CHANGED, 0, 0, 0);
            g_enabled = true;
            updateSubMenus();
        } else {
            MsgBox<>::singleton().set(errMsg::get(10), errMsg::get(11), MB_OK | MB_ICONINFORMATION);
            return;
        }
    } else {
        MsgBox<>::singleton().set(errMsg::get(12),
                                g_msg_mpFss, MB_YESNO | MB_ICONQUESTION, IDYES,
                                binfunctor(&transCevent), ev_options, ngidNormal);
    }                       
}
