/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"
#ifdef FILEID__
#undef FILEID__
#endif
#define FILEID__ hashData_cpp

#include "mpexception.hpp"
#include "globals.hpp"

namespace {
class Csp {
public:
    Csp(LPCTSTR pszContainer = 0, LPCTSTR pszProvider = 0,
            DWORD provType = PROV_RSA_FULL, 
            DWORD flags = CRYPT_VERIFYCONTEXT) : mycsp_(0) {
        if ( CryptAcquireContext(&mycsp_, pszContainer, pszProvider, provType, 
            flags) == FALSE ) {
            throw MPEXCEPTION(GetLastError());
        }
    }
    ~Csp() {
        CryptReleaseContext(mycsp_, 0);
    }
    HCRYPTPROV const & getCsp() const { return mycsp_; }
private:
    explicit Csp(Csp const &) {}
    Csp & operator=(Csp const &) { return *this; }
    HCRYPTPROV mycsp_; 
};

class Hash {
public:
    Hash(Csp & csp, ALG_ID algo = CALG_SHA1, HCRYPTKEY key = 0 ) : hObj_(0) {
        if ( CryptCreateHash(csp.getCsp(), algo, key, 0, &hObj_) == FALSE ) {
            throw MPEXCEPTION(GetLastError());
        }
    }
    ~Hash() {
        CryptDestroyHash(hObj_);
    }
    void addData(BYTE * const data, int const dataSize) const {
        if ( CryptHashData(hObj_, data, dataSize, 0) == FALSE ) {
            throw MPEXCEPTION(GetLastError());
        }
        // TODO remove me
//        throw MPEXCEPTION(GetLastError());
    }
    void getHashedData(BYTE * const outBuf, int * outBufSize) const {
        if ( CryptGetHashParam( hObj_, HP_HASHVAL, outBuf, RC(DWORD*, outBufSize), 0 ) == FALSE ) {
            throw MPEXCEPTION(GetLastError());
        }
    }
private:
    explicit Hash(Hash const &) {}
    Hash & operator=(Hash const &) { return *this; }
    HCRYPTHASH hObj_;
};
} // ns anon

bool hashData(BYTE * data2hash, int data2hashSize, BYTE * hashOut, int hashOutSize) {
    Csp csp;
    Hash hash(csp);
    hash.addData(data2hash, data2hashSize);
    hash.getHashedData(hashOut, &hashOutSize);
    return true;
}

//bool hashData(BYTE * data2hash, int data2hashSize, BYTE * hashOut, int hashOutSize) {
//    HCRYPTHASH hashObj = 0;
//    HCRYPTPROV csp =     0;
//    DWORD bufSize =      hashOutSize;
//    BYTE * buf =         0;
//    bool ret = false;
//
//    if ( CryptAcquireContext(&csp, 0, 0, PROV_RSA_FULL, 
//            CRYPT_VERIFYCONTEXT) == FALSE ) {
//        SOUT("AcquireContext Error:");
//        SOUT(GetLastError());
//        return false;
//    }
//    if ( CryptCreateHash(csp, CALG_SHA1, 0, 0, &hashObj) ) {
//        if ( CryptHashData(hashObj, data2hash, data2hashSize, 0) ) {
//            if ( true /*CryptGetHashParam(hashObj, HP_HASHSIZE, 0, &bufSize, 0)*/ ) {
//                // TODO this next assignment is not ok normal
//                buf = (BYTE*)calloc(1, sizeof(BYTE) * bufSize + 1);          
//                if (buf) {
//                    ret = true;
//                    if ( CryptGetHashParam(hashObj, HP_HASHVAL, buf, &bufSize, 0) ) {
//                        memcpy(hashOut, buf, SC(DWORD, hashOutSize) < bufSize ? hashOutSize : bufSize);
//                    } else {
//                        SOUT("Error getting hash val:");
//                        SOUT(GetLastError());
//                    }
//                    free(buf);
//                    buf = 0;
//                }
//            } else {
//                SOUT("Error getting hash size:");
//                GetLastError();
//            }
//        }
//        
//        CryptDestroyHash(hashObj);
//        hashObj = 0;
//    }
//    CryptReleaseContext(csp, 0);
//
//    return ret;
//}
