/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"

#include "optData.hpp"
#include "hashData.hpp"
#include "mpexception.hpp"
#include "globals.hpp"
#include "MsgBox.hpp"
#include "regKey.hpp"


void inputDev::clear() {
    nameProduct_.clear();
    memset(&guidInstance_, 0, sizeof(GUID));
    axes_ =       0;
    buttons_ =    0;
    activeAxes_ = 0;
}

void inputDev::setButtonsAndAxis(DWORD bu, DWORD ax) { 
    buttons_ =  static_cast<short>(bu);
    axes_ =     static_cast<short>(ax);
}

TCHAR const optData::optFile_[] = _T("\\brakeFixOptions.dat");
std::tstring optData::optFilePath_;

optData::optData() : actDev_(), version_(1U, 0U, 0),
        Ldeadzone_(0), Rdeadzone_(0), LdzSub_(0), RdzSub_(0), LdzRem_(0), RdzRem_(0),
        actDevNr_(0), Linverted_(false), Rinverted_(false), optionsSet_(false),
        canLoadSave_(true) {
    memset(Laxis_, 0, sizeof(Laxis_));
    memset(Raxis_, 0, sizeof(Raxis_));
    memset(&actDev_, 0, sizeof(GUID));
    std::tstring errPath;

    try {
        errPath.reserve(1024);
        errPath.assign(errMsg::get(13));
        RegKey rkey(HKEY_CURRENT_USER,
                    _T("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders"),
                    KEY_READ);
        rkey.enumSubKeys(errPath);
        rkey.getValue(_T("AppData"), optFilePath_);
    } catch (mpexception & e) {
        try {
            errPath.insert(0, e.what(false));
            MessageBox(0, errPath.c_str(), g_msg_mpFssErr, MB_OK | MB_ICONWARNING | MB_TASKMODAL);
        } catch (...) {
            MessageBox(0, e.what(false, errPath.c_str()), g_msg_mpFssErr, 
                       MB_OK | MB_ICONWARNING | MB_TASKMODAL);
        }
        canLoadSave_ = false;
    } catch (std::exception &) {
        MessageBox(0, errMsg::get(13), g_msg_mpFssErr, MB_OK | MB_ICONWARNING | MB_TASKMODAL);
        canLoadSave_ = false;
    }

    optFilePath_.append( _T("\\mpFSS") );
}

optData::~optData() {}


template<typename streamT, typename funcT>
void optData::doReadWrite( streamT & stream, funcT bf, bool read ) {
    short const ds =        sizeof(optData) - sizeof(char);
    short const hSize =     20;
    VersionNumber act(version_);
    char hashMem[hSize] =   { 0 };
    char hashFile[hSize] =  { 0 };
    char * writeP =         RC(char *, &version_);

    if ( read ) { // all those gazillion if(read) suck... maybe we should get rid of this func
        bf( stream, writeP, ds );
    }

    hashData(RC(BYTE*, writeP), ds, RC(BYTE*, hashMem), hSize);

    if ( read ) {
        bf( stream, hashFile, hSize);
        if ( memcmp(&act, &version_, sizeof(VersionNumber)) || memcmp(hashMem, hashFile, hSize) ) {
            MessageBox(0, errMsg::get(14), g_msg_mpFssErr, MB_OK | MB_ICONWARNING | MB_TASKMODAL);
            setOptionsSet(false);
            version_ = act;
            stream.clear();
            stream.close();
            writeOptFile();
        }
    } else {
        bf( stream, writeP, ds );
        bf( stream, hashMem, hSize );
    }    
}

cpccstr    optData::getAxisStr(bool const left) const {
    char const  bufsize = 20;
    char        padding[3];
    size_t      convChars;
    static char str[bufsize];
    TCHAR       tstr[bufsize];
    memset(str, 0, sizeof(str));
    memset(tstr, 0, sizeof(tstr));
    StringCbPrintf(tstr, sizeof(tstr), _T("joystick:%u:%sAxis"), 
            static_cast<uint>(actDevNr_), left ? Laxis_ : Raxis_);
#ifdef UNICODE
    wcstombs_s(&convChars, str, bufsize, tstr, bufsize);
#else
    (convChars);
    memcpy( str, tstr, sizeof(str) );
#endif
    (padding);
    return str;
}

cpctstr    optData::getAxis(bool const left) const {
    return left ? Laxis_ : Raxis_;
}

cpctstr    optData::getDeadzonePercent(bool const left) const {
    static TCHAR str[4] = { 0 };
    double tmp = getDzSub(left) / 655.36;
    if ( !(tmp - static_cast<int>(tmp) < 0.5) ) {
        tmp += 1;
    }
    StringCbPrintf(str, sizeof(str), _T("%d"), static_cast<int>(tmp) );
    return str;
}

int optData::getDeadzone(bool const left) const {
    return left ? Ldeadzone_ : Rdeadzone_;
}

int optData::getDzSub(bool const left) const {
    return left ? LdzSub_ : RdzSub_;
}

int optData::getDzRem(bool const left) const {
    return left ? LdzRem_ : RdzRem_;
}

bool optData::getInverted(bool const left) const {
    return left ? Linverted_ : Rinverted_;
}

GUID const & optData::getGUID() const {
    return actDev_;
}

void optData::setOptionsSet(bool const yes) {
    optionsSet_ = yes;
    if ( yes == false ) {
        memset(&actDev_, 0, sizeof(GUID));
        memset(Laxis_, 0, sizeof(Laxis_));
        memset(Raxis_, 0, sizeof(Raxis_));
        Ldeadzone_ = Rdeadzone_ = LdzSub_ = RdzSub_ = LdzRem_ = RdzRem_ = 0;
        actDevNr_ = SC(char, 0);
        Linverted_ = Rinverted_ = false;
    }
}

bool    optData::areOptionsSet() const {
    return optionsSet_;
}

void    optData::setActDevNr(char const n) {
    actDevNr_ = n;
}

void    optData::setAll( GUID const & in_actDev, cpctstr in_Ldead,
                         cpctstr in_Rdead, int const in_actDevNr,
                         cpctstr in_Laxis, cpctstr in_Raxis,
                         int const in_Linverted, int const in_Rinverted ) {
    optionsSet_ =   true;
    actDev_ =       in_actDev;
    actDevNr_ =     static_cast<uchar>(in_actDevNr);
    Linverted_ =    in_Linverted ? true : false;
    Rinverted_ =    in_Rinverted ? true : false;
    setDeadzone(in_Ldead);
    setDeadzone(in_Rdead, false);
    memcpy(Laxis_, in_Laxis, sizeof(TCHAR)*3);
    memcpy(Raxis_, in_Raxis, sizeof(TCHAR)*3);
}

void    optData::setDeadzone(cpctstr str, bool const left) {
    int tmp = 0;
#ifdef UNICODE
    size_t charsConverted;
    char ansibuf[AXISSTRSIZE * sizeof(TCHAR)] = { 0 };
    wcstombs_s(&charsConverted, ansibuf, sizeof(ansibuf), str, sizeof(ansibuf));
    tmp = atoi(ansibuf);
#else
    tmp = atoi(str);
#endif
    tmp = tmp * PEDALMAXCOMBINED / 100;
    if ( left ) {
        LdzSub_ = tmp;
        LdzRem_ = PEDALMAXCOMBINED - tmp;
    } else {
        RdzSub_ = tmp;
        RdzRem_ = PEDALMAXCOMBINED - tmp;
    }
    tmp = PEDALMIN + tmp;
    if ( left ) {
        Ldeadzone_ = tmp;
    } else {
        Rdeadzone_ = tmp;
    }
}

void    optData::readOptFile() {
    std::ifstream fin;
    std::tstring concat;
    VersionNumber act(version_);
    if ( canLoadSave_ == false) {
        return;
    }
    try {
        concat.reserve( sizeof(optFile_) / sizeof(TCHAR) + optFilePath_.size() );
        concat.assign(optFilePath_);
        concat.append(optFile_);
    } catch (std::exception &) {
        MessageBox(g_winHWND, errMsg::get(15), g_msg_mpFssErr, MB_OK | MB_ICONERROR);
        return;
    }
    fin.open(concat.c_str(), std::ios::binary | std::ios::_Nocreate | std::ios::beg);
    if ( !fin.good() ) {
        return;
    }

    typedef mem_fun2_ref_t<std::istream, std::istream &, char *, std::streamsize> istreamfun;
    try {
        doReadWrite<std::ifstream, istreamfun>( fin, istreamfun(&std::istream::read), true );
    } catch (mpexception & exce) {
        MessageBox(0, exce.what(false, errMsg::get(16)), 
                   g_msg_mpFssErr, MB_OK | MB_ICONERROR | MB_TASKMODAL);
        setOptionsSet(false);
        version_ = act;
        fin.clear();
        fin.close();
        writeOptFile();
        return;
    }
    
    if (actDev_.Data2) {
        setOptionsSet(true);
    }
    fin.clear();
    fin.close();
}

void    optData::writeOptFile() {
    std::ofstream fout;
    std::tstring realOptionsFile;
    std::tstring tmpOptionsFile;
    if ( canLoadSave_ == false) {
        return;
    }
    DWORD fileAtt;
    char const warnText[]= "\r\n\r\nDO NOT EDIT THIS FILE!\r\n"
                           "Feel free to delete if you want to remove your prefs.";

    try {
        realOptionsFile.reserve( sizeof(optFile_) / sizeof(TCHAR) + optFilePath_.size() );
        realOptionsFile.assign(optFilePath_);
        realOptionsFile.append(optFile_);
        tmpOptionsFile.assign(realOptionsFile);
        tmpOptionsFile.append(_T(".tmp"));
    } catch (std::exception &) {
        MessageBox(g_winHWND, errMsg::get(20), g_msg_mpFssErr,
            MB_OK | MB_ICONERROR | MB_TASKMODAL);
        return;
    }
    
    fileAtt = GetFileAttributes( optFilePath_.c_str() );
    if ( fileAtt == INVALID_FILE_ATTRIBUTES ) {
        CreateDirectory(optFilePath_.c_str(), 0);
    } else if ( !(fileAtt & FILE_ATTRIBUTE_DIRECTORY) ) {
        try {
            MessageBox(g_winHWND, std::tstring( errMsg::get(17) ).append(
                optFilePath_).c_str(), g_msg_mpFssErr, MB_OK | MB_ICONERROR);
        } catch (std::exception &) {
            MessageBox(g_winHWND, errMsg::get(17), g_msg_mpFssErr, MB_OK | MB_ICONERROR);
        }
        return;
    }

    fout.open(tmpOptionsFile.c_str(), std::ios::binary | std::ios::beg);
    if (!fout.good()) {
        if ( MessageBox(0, errMsg::get(18), g_msg_mpFssErr, 
                        MB_RETRYCANCEL | MB_TASKMODAL) == IDRETRY ) {
            writeOptFile();
        }
        return;
    }

    typedef mem_fun2_ref_t<std::ostream, std::ostream &, char const *, std::streamsize> ostreamfun;
    try {
        doReadWrite<std::ofstream, ostreamfun>( fout, ostreamfun(&std::ostream::write), false );   
        fout.write(warnText, sizeof(warnText)-1);
    } catch (mpexception & exce) {
        fout.clear(); 
        fout.close();
        DeleteFile(tmpOptionsFile.c_str());
        if ( MessageBox(0, exce.what(false, errMsg::get(18)), g_msg_mpFssErr, 
                        MB_RETRYCANCEL | MB_TASKMODAL) == IDRETRY ) {
            writeOptFile();
        }
        return;
    }

    fout.flush();
    fout.clear();
    fout.close();

    BOOL rt;
    rt = MoveFileEx(tmpOptionsFile.c_str(), realOptionsFile.c_str(), MOVEFILE_REPLACE_EXISTING);
    if ( rt == FALSE ) {
        MessageBox(0, errMsg::get(19), g_msg_mpFssErr, MB_OK | MB_ICONERROR);
    }
}
