/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krüger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"

#include "globals.hpp"
#include "dxi.hpp"
#include "optDialog.hpp"
#include "Resource.h"
#include "mpexception.hpp"
#include "MsgBox.hpp"

INT_PTR CALLBACK optDialogCB(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
    switch (message) {
    case WM_COMMAND:
        switch ( HIWORD(wParam) ) {
        case CBN_SELCHANGE:
            if ( reinterpret_cast<HWND>(lParam) == g_optsDialog->ComboDev_ ) {
                g_optsDialog->onComboDevChange();
            }
            return (INT_PTR)TRUE;
        case EN_CHANGE:
            checkTxt(reinterpret_cast<HWND>(lParam));
            return (INT_PTR)TRUE;
        case BN_CLICKED:
            if ( reinterpret_cast<HWND>(lParam) == g_optsDialog->Linverted_ ||
                reinterpret_cast<HWND>(lParam) == g_optsDialog->Rinverted_ ) {
                Button_SetCheck(reinterpret_cast<HWND>(lParam), 
                                !Button_GetCheck(reinterpret_cast<HWND>(lParam)));
                return (INT_PTR)TRUE;
            }
        }
        switch( LOWORD(wParam) ) {
        case IDOK:
            g_optsDialog->onOk();
            return (INT_PTR)TRUE;
        case IDCANCEL:
            EndDialog(hDlg, 1);
            return (INT_PTR)TRUE;
        }
        break;
    case WM_INITDIALOG:
        g_optsDialog->OnInit(hDlg);
        return (INT_PTR)TRUE;
    }
    return (INT_PTR)FALSE;
}

namespace {

void checkTxt(HWND const field) {
    ushort txtSel = LOWORD(Edit_GetSel(field));
    TCHAR  txt[TXTSIZE] = { 0 };
    if ( !Edit_GetModify(field) ) { return; }
    Edit_GetText(field, txt, TXTSIZE);
    if ( txt[1] > '9' || txt[1] < '0' ) {
        txt[1] = 0;
    }
    if ( txt[0] > '9' || txt[0] < '0' ) {
        txt[0] = txt[1];
        txt[1] = 0;
    }
    Edit_SetText(field, txt);
    Edit_SetSel(field, txtSel, txtSel);
}


} // ns anon

optDialog::optDialog() 
        : hDlg_(0),ComboDev_(0),Laxis_(0),Linverted_(0),Ldeadzone_(0),Raxis_(0),
        Rinverted_(0),Rdeadzone_(0),ComboDevSelection_(-1),inputDevs_() {
    inputDevs_.clear();
    inputDevs_.reserve(4);
}

bool optDialog::show() const {
    if (DialogBox(g_hInst, MAKEINTRESOURCE(IDD_DIALOG), g_winHWND, optDialogCB) == -1 ) {
        TCHAR buf[256] = {0};
        StringCbPrintf(buf, sizeof(buf), errMsg::get(21), GetLastError());
        MessageBox(g_winHWND, buf, g_msg_mpFssErr, MB_OK | MB_ICONERROR);
        return false;
    } else {
        return true;
    }
}

void optDialog::enableOtherControls(const bool enable) const {
    ComboBox_Enable(Laxis_, enable);
    ComboBox_Enable(Raxis_, enable);
    Edit_Enable(Ldeadzone_, enable);
    Edit_Enable(Rdeadzone_, enable);
    Button_Enable(Linverted_, enable);
    Button_Enable(Rinverted_, enable);
}

void optDialog::onComboDevChange() {
    ComboDevSelection_ = ComboBox_GetCurSel(ComboDev_);
    if (ComboDevSelection_) {
        enableOtherControls(true);
        setAxisPulldown(Laxis_, ComboDevSelection_);
        setAxisPulldown(Raxis_, ComboDevSelection_);
    } else {
        enableOtherControls(false);
    }
}

void optDialog::OnInit(const HWND in_hDlg) {
    inputDevVec::const_iterator act;
    inputDevVec::const_iterator end;
    cpctstr textNoDevs = _T("No device selected");
    hDlg_ =         in_hDlg;
    ComboDev_ =     GetDlgItem(hDlg_, DIALOG_ComboDev);
    Laxis_ =        GetDlgItem(hDlg_, DIALOG_Laxis);
    Linverted_ =    GetDlgItem(hDlg_, DIALOG_Linverted);
    Ldeadzone_ =    GetDlgItem(hDlg_, DIALOG_Ldeadzone);
    Raxis_ =        GetDlgItem(hDlg_, DIALOG_Raxis);
    Rinverted_ =    GetDlgItem(hDlg_, DIALOG_Rinverted);
    Rdeadzone_ =    GetDlgItem(hDlg_, DIALOG_Rdeadzone);
    try {
        enumDevices(inputDevs_);
    } catch (mpexception & e) {
        EndDialog(hDlg_, 1);
        MessageBox(g_winHWND, e.what(false), g_msg_mpFssErr, MB_OK | MB_ICONERROR);
        return;
    }
    SendMessage(ComboDev_, CB_ADDSTRING, 0, (LPARAM)textNoDevs);
    act = inputDevs_.begin();
    end = inputDevs_.end();
    if (g_options->areOptionsSet()) {
        for ( short index = 1; act != end; ++act, ++index ) {
            ComboBox_AddString(ComboDev_, act->getNameProduct().c_str());
            if ( g_options->getGUID() == act->getGUID() ) {
                ComboBox_SetCurSel(ComboDev_, index);
                ComboDevSelection_ = ComboBox_GetCurSel(ComboDev_);
                setAxisPulldown(Laxis_, ComboDevSelection_);
                setAxisPulldown(Raxis_, ComboDevSelection_);
                ComboBox_SelectString(Laxis_, -1, g_options->getAxis());
                ComboBox_SelectString(Raxis_, -1, g_options->getAxis(false));
                Button_SetCheck(Linverted_, g_options->getInverted());
                Button_SetCheck(Rinverted_, g_options->getInverted(false));
                Edit_SetText(Ldeadzone_, g_options->getDeadzonePercent());
                Edit_SetText(Rdeadzone_, g_options->getDeadzonePercent(false));
            }
        }
        if ( ComboBox_GetCurSel(ComboDev_) == CB_ERR ) {
            MessageBox(g_winHWND, errMsg::get(22), errMsg::get(11), MB_OK | MB_ICONINFORMATION);
            ComboBox_SelectString(ComboDev_, -1, textNoDevs);
            Edit_SetText(Ldeadzone_, _T("0"));
            Edit_SetText(Rdeadzone_, _T("0"));
            enableOtherControls(false);
        }
    } else {
        for ( ; act != end; ++act ) {
            ComboBox_AddString(ComboDev_, act->getNameProduct().c_str());
        }
        ComboBox_SelectString(ComboDev_, -1, textNoDevs);
        Edit_SetText(Ldeadzone_, _T("0"));
        Edit_SetText(Rdeadzone_, _T("0"));
        enableOtherControls(false);
    }
    Edit_LimitText(Ldeadzone_, 2);
    Edit_LimitText(Rdeadzone_, 2);
}

void optDialog::onOk() {
    TCHAR LdzText[TXTSIZE] =    { 0 };
    TCHAR RdzText[TXTSIZE] =    { 0 };
    TCHAR LaxisText[TXTSIZE] =    { 0 };
    TCHAR RaxisText[TXTSIZE] =    { 0 };
    ComboDevSelection_ = ComboBox_GetCurSel(ComboDev_);
    if ( ComboDevSelection_ ) {
        // this has to change to just an error if we dont use dx to determine the input
        // with fsx this is not possible..you cannot assign the same axis to 2 things
        if ( ComboBox_GetCurSel(Laxis_) == ComboBox_GetCurSel(Raxis_) ) {
        //    if ( MessageBox(g_winHWND, _T("The brake axes are the same.")
        //            _T("\nIs this what you intended?"), _T("Axes identical"),
        //            MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDNO ) {
        //        return;
        //    }
            MessageBox(hDlg_, errMsg::get(24), g_msg_mpFss, MB_OK | MB_ICONINFORMATION);
            return;
        }

        Edit_GetText(Ldeadzone_, LdzText, TXTSIZE);
        Edit_GetText(Rdeadzone_, RdzText, TXTSIZE);
        ComboBox_GetText(Laxis_, LaxisText, TXTSIZE);
        ComboBox_GetText(Raxis_, RaxisText, TXTSIZE);
        g_options->setAll(inputDevs_.at(ComboDevSelection_-1).getGUID(), LdzText, RdzText, 
                    ComboDevSelection_-1, LaxisText, RaxisText, 
                    Button_GetCheck(Linverted_), Button_GetCheck(Rinverted_));
    } else {
        g_options->setOptionsSet(false);
    }
    g_options->writeOptFile();
    EndDialog(hDlg_, TRUE);
}

void optDialog::setAxisPulldown(const HWND pd, const int Selec) const {
    ComboBox_ResetContent(pd);
    uint const actAxes = inputDevs_.at(Selec-1).getActiveAxes();
    if ( actAxes & BIT1 ) {
        ComboBox_AddString(pd, _T("X"));
    }
    if ( actAxes & BIT2 ) {
        ComboBox_AddString(pd, _T("Y"));
    }    
    if ( actAxes & BIT3 ) {
        ComboBox_AddString(pd, _T("Z"));
    }
    if ( actAxes & BIT4 ) {
        ComboBox_AddString(pd, _T("Rx"));
    }
    if ( actAxes & BIT5 ) {
        ComboBox_AddString(pd, _T("Ry"));
    }    
    if ( actAxes & BIT6 ) {
        ComboBox_AddString(pd, _T("Rz"));
    }
    ComboBox_SetCurSel(pd, 0);
}
