/*  macpal flight simulator stuff brakeFix is a plugin for Microsoft Flight
    Simulator X and is released under the LGPL v3 (or later). It corrects 
    the brake output to make an autobrake working.
    Copyright 2009, 2010 Knud Krger slomo@mpfss.de

    This file is part of mpFSS.brakeFix.

    mpFSS.brakeFix is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mpFSS.brakeFix is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mpFSS.brakeFix.  If not, see <http://www.gnu.org/licenses/>. */
#include "stdafx.h"

#ifdef FILEID__
#undef FILEID__
#endif
#define FILEID__ regKey_cpp

#include "regKey.hpp"
#include "mpexception.hpp"

RegKey::RegKey(HKEY rootKey, LPCTSTR subKey, REGSAM sam) : key_(0) {
    LONG rt;
    rt = RegOpenKeyEx(rootKey, subKey, 0,
                      sam | KEY_ENUMERATE_SUB_KEYS | KEY_QUERY_VALUE, &key_);
    if ( rt != ERROR_SUCCESS ) {
        throw MPEXCEPTION(GetLastError());
    }
}

RegKey::~RegKey() {
    if ( key_ != HKEY_CLASSES_ROOT &&
         key_ != HKEY_CURRENT_USER &&
         key_ != HKEY_LOCAL_MACHINE &&
         key_ != HKEY_USERS ) {
        RegCloseKey(key_);
    }
}

// can throw mpexception and std::bad_alloc
void RegKey::getValue(LPCTSTR valueName, std::tstring & outStr) const {
    DWORD dataSize = 0;
    DWORD tcharSize = 0;
    LONG rt;
    boost::scoped_array<TCHAR> data;
    rt = RegQueryValueEx(key_, valueName, 
                         0,                 // reserved 
                         0,                 // opt outType
                         0,                 // opt outData
                         &dataSize);
    if ( rt != ERROR_SUCCESS || !dataSize) {
        throw MPEXCEPTION(rt);
    }
    
    dataSize += sizeof(TCHAR);
    tcharSize = dataSize / sizeof(TCHAR);
    data.reset(new TCHAR[tcharSize]);
    memset(data.get(), 0, dataSize);
    
    rt = RegQueryValueEx(key_, valueName, 
                         0,                 // reserved 
                         0,                 // opt outType
                         RC(BYTE*, data.get()),   // opt outData
                         &dataSize);
    if ( rt != ERROR_SUCCESS ) {
        throw MPEXCEPTION(rt);
    }

    outStr.assign(data.get());
}

// can throw std::bad_alloc
void RegKey::enumSubKeys(std::tstring & outStr) const {
    enum { bufSizeConst = 256 };
    DWORD idx =             0;
    DWORD bufSize =         bufSizeConst;
    TCHAR buf[bufSizeConst];
    buf[bufSizeConst-1] =   0;
    LONG rt;
    do {
        rt = RegEnumValue(key_, idx, buf, &bufSize, 0, 0, 0, 0);
        if ( rt == ERROR_SUCCESS ) {
            outStr.append(buf);
            outStr.append(_T("\n"));
            buf[0] = 0;
        }
        bufSize = bufSizeConst;
        ++idx;
    } while ( rt != ERROR_NO_MORE_ITEMS ); 
}
